;---------------------------------------------------------------------------
;  S A V E  /  R E S T O R E    W O R K S P A C E    E N V I R O N M E N T S
;---------------------------------------------------------------------------
;
;   These procedures are designed to allow you to Save/Restore a given
; work-environment onto the workspace.
;
;   It solves the problem where you may be in one form-view, then change
; to another master-table and form, and need to return to the original
; environment.  You COULD have each and every procedure either return
; the environment to the way it found it, or you could have all procedures
; deliberately force the environment into the proper settings, or you
; can just use these two procedures to do it for you.
;    Also note that if you have a few "common" workspaces for the bulk of
; your application, you could manually bring each one up in turn and save
; them, then just call Environment.Save (<target environment>).  This is
; significantly easier than firing off all of the FormKey's, MoveTo's and
; pickform's all over your code.
;    The procedures are intended for simply switching between various views,
; and do not have the ability to place missing tables back onto the workspace
; if they were cleared between the Save and Restore.
;
;    Note that this also can flip between CoEdit and Main mode, but no
; other system modes are handled.  You are free to extend that functionality
; to include automatically going from any two states you wish.  If you do so,
; please be kind enough to let me know, as I would like to add that
; functionality too, but have not had the time.
;
;  Written by Adam Guberman                Released into public domain.
;             Mindware Solutions, Inc.
;             11709 Bowman Green Drive
;             Reston, VA  22090
;             CIS: 72310,2274
;-----------------------------------------------------------------------
; To use save an environment:
;    1. Declare a dynarray to hold workspace data
;    2. Call Environment.Save ( <your dynarray> )
; To return to that environment:
;    3. Call Environment.Restore  ( <your dynarray> )
;-----------------------------------------------------------------------


;-----------------------------------  ENVIRONMENT.Save -----------------
; Saves much of the curent workspace environment for future re-use
Proc environment.Save (DVar)

   if ImageType() <> "Display" Then
      Message "Can not Save workspace when not in a display image..."
      beep
      sleep 2000
      DVar["Error"] = ImageType()
      Return False
   EndIf

   Dvar["Table"]    = Table()
   if IsFormView() Then
      MoveTo MasterImageNo()               ;
      Dvar["MasterTable"]    = Table()     ; grab parent table
      DVar["FormView"]       = form()      ; grab the form
   Else
      DVar["FormView"] = 0
   EndIf
   DVar["SysMode"] = SysMode ()
   Return True
EndProc



;-----------------------------------  ENVIRONMENT.Restore  -----------------
; Restores workspace to a given environment
Proc environment.Restore (DVar)

     if IsAssigned(DVar["Error"]) Then
        Message "The environment was not properly saved... restore not possible."
        beep
        Sleep 2000
        Return False
     EndIf

     if ImageType() <> "Display" Then
        Message "You need to be in a display image to restore."
        Sleep 2000
        Beep
        Return False
     EndIf

     if IsFormView() Then
        FORMKEY
     EndIf

     if IsAssigned(DVar["MasterTable"]) Then    ; was originally in a form
        MoveTo DVar["MasterTable"]
        Pickform DVar["FormView"]
     EndIf

     MoveTo DVar["Table"]

     Switch
        Case DVar["SysMode"] = "CoEdit" and SysMode()="Main" : CoEditKey
        Case DVar["SysMode"] = "Main" and SysMode()="CoEdit" : Do_It!
     EndSwitch

     Return True
EndProc
